@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import asyncio
 import concurrent.futures
 import logging
@@ -8,15 +9,20 @@ import stat
 import threading
 import time
 from collections.abc import Iterable, Sequence
-from pickle import dumps as pickle_dumps, loads as pickle_loads
+from pickle import dumps as pickle_dumps
+from pickle import loads as pickle_loads
 from time import perf_counter
-from typing import Callable, Awaitable, Any, Optional
+from typing import Any, Awaitable, Callable, Optional
 
 from overrides import EnforceOverrides
 
-from .util import get_docker_host_ip, is_exception, is_async, MAX_THREADS
-from .util import RemoteException
 from ._queues import SingleLane
+from .util import (
+    MAX_THREADS,
+    RemoteException,
+    get_docker_host_ip,
+    is_async,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -498,7 +504,7 @@ class SocketClient(EnforceOverrides):
             z = q.get()
             if z == "OK":
                 n += 1
-            if is_exception(z):
+            if isinstance(z, BaseException):
                 self._to_shutdown.set()
                 concurrent.futures.wait(self._tasks)
                 self._executor.shutdown()
@@ -564,7 +570,7 @@ class SocketClient(EnforceOverrides):
                 # Do not capture `asyncio.IncompleteReadError`;
                 # let it stop this function.
                 fut = active.pop(req_id)
-                if is_exception(data):
+                if isinstance(data, BaseException):
                     fut.set_exception(data)
                 else:
                     fut.set_result(data)
