@@ -39,31 +39,36 @@ from __future__ import annotations
 # that defines that class itself.
 # https://stackoverflow.com/a/49872353
 # Will no longer be needed in Python 3.10.
-
 import concurrent.futures
+import functools
 import os
 import queue
 import threading
 import traceback
 from collections import deque
-from collections.abc import Iterable, Iterator, Sequence
+from collections.abc import Iterable, Sequence
 from multiprocessing.util import Finalize
 from random import random
 from typing import (
-    Callable,
-    TypeVar,
-    Optional,
     Any,
+    Callable,
     Literal,
+    Optional,
+    TypeVar,
 )
 
 from deprecation import deprecated
-from overrides import EnforceOverrides, overrides, final
 
-from .util import is_remote_exception, get_remote_traceback
-from .util import is_exception, Thread, MP_SPAWN_CTX
 from ._queues import SingleLane
-
+from .util import (
+    MAX_THREADS,
+    MP_SPAWN_CTX,
+    Thread,
+    get_remote_traceback,
+    get_shared_process_pool,
+    get_shared_thread_pool,
+    is_remote_exception,
+)
 
 FINISHED = "8d906c4b-1161-40cc-b585-7cfb012bca26"
 STOPPED = "ceccca5e-9bb2-46c3-a5ad-29b3ba00ad3e"
@@ -75,57 +80,35 @@ T = TypeVar("T")  # indicates input data element
 TT = TypeVar("TT")  # indicates output after an op on `T`
 
 
-class Streamer(EnforceOverrides, Iterator):
+class Stream(Iterable):
     """
-    The class ``Streamer`` is the "entry-point" for the "streamer" utilities.
-    User constructs a ``Streamer`` object
+    The class ``Stream`` is the "entry-point" for the "streamer" utilities.
+    User constructs a ``Stream`` object
     by passing an `Iterable`_ to it, then calls its methods to use it.
     """
 
-    def __init__(self, instream: Iterator | Iterable, /):
+    def __init__(self, instream: Iterable, /):
         """
         Parameters
         ----------
         instream
-            The input stream of elements. This is an `Iterable`_ or an `Iterator`_,
-            which could be unlimited.
+            The input stream of elements. This is a possibly unlimited  `Iterable`_.
         """
-        self.streamlets: list[Stream] = [Stream(instream)]
-
-    def stop(self):
-        """
-        Stop all the operators that are working on the stream.
-
-        Usually you do not need to call this method unless you break out of
-        iteration prematurely or exception happens. But when those situations are possible,
-        it's advised to consume this stream within a context manager, which calls
-        ``stop`` upon exit.
-        """
-        for s in self.streamlets:
-            s._stop()
+        self.streamlets: list[Stream] = [instream]
 
+    @deprecated(
+        deprecated_in="0.11.9",
+        removed_in="0.12.2",
+        details="Please use the object directly without context manager.",
+    )
     def __enter__(self):
-        """
-        If you break out of iteration before it finishes, or error happens, then it's important
-        to use this object within the context manager.
-        If these two situations do not arise, then it's not necessary to use context manager.
-
-        Calls to the "operation setup" methods do not need to be within context manager.
-        Only the "consuming" methods (:meth:`__iter__`, :meth:`__next__`, and :meth:`drain`)
-        need to be within context manager.
-        """
         return self
 
     def __exit__(self, *args, **kwargs):
-        self.stop()
+        pass
 
-    @final
     def __iter__(self):
-        return self
-
-    @final
-    def __next__(self):
-        return self.streamlets[-1].__next__()
+        yield from self.streamlets[-1]
 
     def drain(self) -> int:
         """Drain off the stream and return the number of elements processed.
@@ -281,7 +264,7 @@ class Streamer(EnforceOverrides, Iterator):
         """
 
         def foo(x):
-            if is_exception(x):
+            if isinstance(x, BaseException):
                 if keep_exc_types is not None and isinstance(x, keep_exc_types):
                     return True
                 if drop_exc_types is not None and isinstance(x, drop_exc_types):
@@ -295,7 +278,7 @@ class Streamer(EnforceOverrides, Iterator):
         self,
         *,
         print_func: Optional[Callable[[str], None]] = None,
-        interval: int | float = 1000,
+        interval: int | float = 1,
         exc_types: Optional[Sequence[type[BaseException]]] = BaseException,
         with_exc_tb: bool = True,
     ):
@@ -368,13 +351,13 @@ class Streamer(EnforceOverrides, Iterator):
                 if (
                     not should_print
                     and self._exc_types
-                    and is_exception(x)
+                    and isinstance(x, BaseException)
                     and isinstance(x, self._exc_types)
                 ):
                     should_print = True
                 if not should_print:
                     return x
-                if not is_exception(x):
+                if not isinstance(x, BaseException):
                     self._print_func("#%d:  %r" % (self._idx, x))
                     return x
                 trace = ""
@@ -395,7 +378,7 @@ class Streamer(EnforceOverrides, Iterator):
         return self.map(Peeker())
 
     @deprecated(
-        deprecated_in="0.11.8", removed_in="0.13.0", details="Use ``peek`` instead."
+        deprecated_in="0.11.8", removed_in="0.12.2", details="Use ``peek`` instead."
     )
     def peek_every_nth(self, n: int):
         return self.peek(interval=n)
@@ -442,7 +425,7 @@ class Streamer(EnforceOverrides, Iterator):
         Examples
         --------
         >>> data = ['atlas', 'apple', 'answer', 'bee', 'block', 'away', 'peter', 'question', 'plum', 'please']
-        >>> print(Streamer(data).groupby(lambda x: x[0]).collect())
+        >>> print(Stream(data).groupby(lambda x: x[0]).collect())
         [['atlas', 'apple', 'answer'], ['bee', 'block'], ['away'], ['peter'], ['question'], ['plum', 'please']]
         """
         self.streamlets.append(Grouper(self.streamlets[-1], func, **kwargs))
@@ -463,24 +446,12 @@ class Streamer(EnforceOverrides, Iterator):
 
         Examples
         --------
-        >>> ss = Streamer(range(10)).batch(3)
+        >>> ss = Stream(range(10)).batch(3)
         >>> print(ss.collect())
         [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
         """
-
-        class Batcher:
-            def __init__(self):
-                self._n = 0
-                self._z = 0
-
-            def __call__(self, x, batch_size):
-                if self._n == batch_size:
-                    self._z = 1 - self._z
-                    self._n = 0
-                self._n += 1
-                return self._z
-
-        return self.groupby(Batcher(), batch_size=batch_size)
+        self.streamlets.append(Batcher(self.streamlets[-1], batch_size))
+        return self
 
     def unbatch(self):
         """
@@ -496,7 +467,7 @@ class Streamer(EnforceOverrides, Iterator):
         ...     if isinstance(x, int) and x > 0:
         ...         return [x] * x
         ...     return []
-        >>> ss = Streamer([0, 1, 2, 'a', -1, 'b', 3, 4]).map(explode).unbatch()
+        >>> ss = Stream([0, 1, 2, 'a', -1, 'b', 3, 4]).map(explode).unbatch()
         >>> print(ss.collect())
         [1, 2, 2, 3, 3, 3, 4, 4, 4, 4]
 
@@ -507,7 +478,7 @@ class Streamer(EnforceOverrides, Iterator):
         ...     for _ in range(n):
         ...         yield n
         >>>
-        >>> stream = Streamer((1, 2, 4, 3, 0, 5)).map(expand).unbatch().collect()
+        >>> stream = Stream((1, 2, 4, 3, 0, 5)).map(expand).unbatch().collect()
         >>> print(stream)
         [1, 2, 2, 4, 4, 4, 4, 3, 3, 3, 5, 5, 5, 5, 5]
         """
@@ -550,7 +521,7 @@ class Streamer(EnforceOverrides, Iterator):
 
         Examples
         --------
-        >>> ss = Streamer(range(7))
+        >>> ss = Stream(range(7))
         >>> print(ss.accumulate(lambda x, y: x + y, 3).collect())
         [3, 4, 6, 9, 13, 18, 24]
         """
@@ -588,7 +559,7 @@ class Streamer(EnforceOverrides, Iterator):
         return self
 
     @deprecated(
-        deprecated_in="0.11.8", removed_in="0.13.0", details="Use ``parmap`` instead"
+        deprecated_in="0.11.8", removed_in="0.12.2", details="Use ``parmap`` instead"
     )
     def transform(
         self,
@@ -657,10 +628,8 @@ class Streamer(EnforceOverrides, Iterator):
             that can be ongoing at any time.
 
             If ``None``, a default value is used depending on the value of ``executor``.
-            If ``executor`` is ``'thread'``, as many as
-            ``min(32, (os.cpu_count() or 1) + 4)`` threads are created.
-            If ``executor`` is ``'process'``, as many as
-            ``concurrency = os.cpu_count() or 1`` processes are created.
+            Unless there are concrete reasons that you need to restrict the level of concurrency
+            or the use of resources, it is recommended to leave `num_workers` at `None`.
         return_x
             If ``True``, output stream will contain tuples ``(x, y)``;
             if ``False``, output stream will contain ``y`` only.
@@ -672,6 +641,8 @@ class Streamer(EnforceOverrides, Iterator):
             Note that a ``True`` value does not absorb exceptions
             raised by *previous* operators in the pipeline; it is concerned about
             exceptions raised by ``func`` only.
+        **kwargs
+            Passed on to :class:`Parmapper`.
         """
         self.streamlets.append(
             Parmapper(
@@ -687,156 +658,113 @@ class Streamer(EnforceOverrides, Iterator):
         return self
 
 
-class Stream(EnforceOverrides):
-    def __init__(self, instream: Stream | Iterator | Iterable, /):
-        if hasattr(instream, "__next__"):
-            self._instream = instream
-        else:
-            self._instream = iter(instream)
-        self._stopped = threading.Event()
+class Mapper(Iterable):
+    def __init__(self, instream: Iterable, func: Callable[[T], Any], **kwargs):
+        self._instream = instream
+        self.func = functools.partial(func, **kwargs) if kwargs else func
 
-    def _stop(self):
-        # Clean up and deal with early-stop situations.
-        self._stopped.set()
-
-    @final
     def __iter__(self):
-        return self
-
-    def __next__(self):
-        """Produce the next element in the stream.
-
-        In a subclass, almost always it will not get the next element
-        from ``self._instream``, but rather from some object that holds
-        results of transformations on ``self._instream``.
-        In other words, subclass typically needs to override this method.
-        Subclass should take care to handle exceptions in this method.
-        """
-        if self._stopped.is_set():
-            raise StopIteration
-        z = next(self._instream)
-        return z
+        func = self.func
+        for v in self._instream:
+            yield func(v)
 
 
-class Mapper(Stream):
-    def __init__(self, instream: Stream, func: Callable[[T], Any], **kwargs):
-        super().__init__(instream)
-        self.func = func
-        self._kwargs = kwargs
+class Filter(Iterable):
+    def __init__(self, instream: Iterable, func: Callable[[T], bool], **kwargs):
+        self._instream = instream
+        self.func = functools.partial(func, **kwargs) if kwargs else func
 
-    @overrides
-    def __next__(self):
-        if self._stopped.is_set():
-            raise StopIteration
-        return self.func(next(self._instream), **self._kwargs)
+    def __iter__(self):
+        func = self.func
+        for v in self._instream:
+            if func(v):
+                yield v
 
 
-class Filter(Stream):
-    def __init__(self, instream: Stream, func: Callable[[T], bool], **kwargs):
-        super().__init__(instream)
-        self.func = func
-        self._kwargs = kwargs
-
-    @overrides
-    def __next__(self):
-        if self._stopped.is_set():
-            raise StopIteration
-        while True:
-            z = next(self._instream)
-            if not self.func(z, **self._kwargs):
-                continue
-            return z
-
-
-class Header(Stream):
-    def __init__(self, instream: Stream, /, n: int):
+class Header(Iterable):
+    def __init__(self, instream: Iterable, /, n: int):
         """
         Keeps the first ``n`` elements and ignores all the rest.
         """
         assert n > 0
-        super().__init__(instream)
+        self._instream = instream
         self.n = n
-        self._n = 0
 
-    @overrides
-    def __next__(self):
-        if self._stopped.is_set():
-            raise StopIteration
-        if self._n >= self.n:
-            raise StopIteration
-        self._n += 1
-        return next(self._instream)
+    def __iter__(self):
+        n = 0
+        nn = self.n
+        for v in self._instream:
+            if n >= nn:
+                break
+            yield v
+            n += 1
 
 
-class Tailor(Stream):
-    def __init__(self, instream: Stream, /, n: int):
+class Tailor(Iterable):
+    def __init__(self, instream: Iterable, /, n: int):
         """
         Keeps the last ``n`` elements and ignores all the previous ones.
         If there are less than ``n`` data elements in total, then keep all of them.
 
+        This object needs to walk through the entire input stream before
+        starting to yield elements.
+
         .. note:: ``n`` data elements need to be kept in memory, hence ``n`` should
             not be "too large" for the typical size of the data elements.
         """
-        super().__init__(instream)
+        self._instream = instream
         assert n > 0
         self.n = n
-        self._data = deque(maxlen=n)
-        self._data_collected = False
 
-    @overrides
-    def __next__(self):
-        if self._stopped.is_set():
-            raise StopIteration
-        if not self._data_collected:
-            data = self._data
-            for v in self._instream:
-                data.append(v)
-            self._data_collected = True
-        try:
-            return self._data.popleft()
-        except IndexError:
-            # No more data.
-            raise StopIteration
+    def __iter__(self):
+        data = deque(maxlen=self.n)
+        for v in self._instream:
+            data.append(v)
+        yield from data
 
 
-class Grouper(Stream):
-    def __init__(self, instream: Stream, /, func: Callable[[T], Any], **kwargs):
-        super().__init__(instream)
-        self._func = func
-        self._kwargs = kwargs
-        self._z = object()
-        self._group = None
+class Grouper(Iterable):
+    def __init__(self, instream: Iterable, /, func: Callable[[T], Any], **kwargs):
+        self._instream = instream
+        self.func = functools.partial(func, **kwargs) if kwargs else func
 
-    @overrides
-    def __next__(self):
-        if self._stopped.is_set():
-            raise StopIteration
-
-        while True:
-            try:
-                x = next(self._instream)
-            except StopIteration:
-                self._stop()
-                break
-            z = self._func(x, **self._kwargs)
-            if z == self._z:
-                self._group.append(x)
+    def __iter__(self):
+        _z = object()
+        group = None
+        func = self.func
+        for x in self._instream:
+            z = func(x)
+            if z == _z:
+                group.append(x)
             else:
-                self._z = z
-                if self._group is None:
-                    self._group = [x]
-                    continue
-                batch = self._group
-                self._group = [x]
-                return batch
-
-        if self._group:
-            return self._group
-        raise StopIteration
+                if group is not None:
+                    yield group
+                group = [x]
+                _z = z
+        if group:
+            yield group
 
 
-class Unbatcher(Stream):
-    def __init__(self, instream: Stream, /):
+class Batcher(Iterable):
+    def __init__(self, instream: Iterable, /, batch_size: int):
+        self._instream = instream
+        assert batch_size > 0
+        self._batch_size = batch_size
+
+    def __iter__(self):
+        batch_size = self._batch_size
+        batch = []
+        for x in self._instream:
+            batch.append(x)
+            if len(batch) == batch_size:
+                yield batch
+                batch = []
+        if batch:
+            yield batch
+
+
+class Unbatcher(Iterable):
+    def __init__(self, instream: Iterable, /):
         """
         The incoming stream consists of lists.
         This object "expands" or "flattens" the lists into a stream
@@ -846,60 +774,51 @@ class Unbatcher(Stream):
         This may correspond to a "Batcher" operator upstream,
         but that is by no means a requirement.
         """
-        super().__init__(instream)
-        self._batch = iter([])
+        self._instream = instream
 
-    @overrides
-    def __next__(self):
-        if self._stopped.is_set():
-            raise StopIteration
-        try:
-            z = next(self._batch)
-        except StopIteration:
-            y = next(self._instream)
-            self._batch = iter(y)
-            return self.__next__()
-        else:
-            return z
+    def __iter__(self):
+        for x in self._instream:
+            yield from x
 
 
-class Buffer(Stream):
-    def __init__(self, instream: Stream, /, maxsize: int):
-        super().__init__(instream)
+class Buffer(Iterable):
+    def __init__(self, instream: Iterable, /, maxsize: int):
+        self._instream = instream
         assert 1 <= maxsize <= 10_000
         self.maxsize = maxsize
-        self._tasks = SingleLane(maxsize)
-        self._worker = Thread(target=self._start, loud_exception=False)
+        self._finalizer_func = None
+        self._loud_exception = False
+
+    def _start(self):
+        self._stopped = threading.Event()
+        self._tasks = SingleLane(self.maxsize)
+        self._worker = Thread(
+            target=self._run_worker, loud_exception=self._loud_exception
+        )
         self._worker.start()
         self._finalize_func = Finalize(
             self,
-            type(self)._finalize,
+            type(self)._finalizer,
             (self._stopped, self._tasks, self._worker),
             exitpriority=10,
         )
 
-    def _start(self):
+    def _run_worker(self):
         threading.current_thread().name = "BufferThread"
         q = self._tasks
-        while True:
-            try:
-                v = next(self._instream)
-            except StopIteration:
-                q.put(FINISHED)
-                break
-            except Exception:
-                q.put(STOPPED)
-                raise
-            if self._stopped.is_set():
-                break
-            q.put(v)  # if `q` is full, will wait here
-
-    @overrides
-    def _stop(self):
-        self._finalize_func()
+        try:
+            for x in self._instream:
+                if self._stopped.is_set():
+                    return
+                q.put(x)  # if `q` is full, will wait here
+        except Exception:
+            q.put(STOPPED)
+            raise
+        else:
+            q.put(FINISHED)
 
     @staticmethod
-    def _finalize(stopped, tasks, worker):
+    def _finalizer(stopped, tasks, worker):
         stopped.set()
         while not tasks.empty():
             _ = tasks.get()
@@ -907,128 +826,196 @@ class Buffer(Stream):
         # more element into the queue, which is safe.
         worker.join()
 
-    @overrides
-    def __next__(self):
-        if self._stopped.is_set():
-            raise StopIteration
+    def _finalize(self):
+        fin = self._finalize_func
+        if fin:
+            self._finalize_func = None
+            fin()
+
+    def __iter__(self):
+        self._start()
+
         while True:
             try:
                 z = self._tasks.get(timeout=1)
-                break
             except queue.Empty as e:
                 if self._worker.is_alive():
                     continue
-                if self._stopped.is_set():
-                    raise StopIteration
                 if self._worker.exception():
                     raise self._worker.exception()
                 raise RuntimeError("unknown situation occurred") from e
-        if z == FINISHED:
-            self._stop()
-            raise StopIteration
-        if z == STOPPED:
-            self._stop()
-            raise self._worker.exception()
-        return z
+            else:
+                if z == FINISHED:
+                    break
+                if z == STOPPED:
+                    raise self._worker.exception()
+                try:
+                    yield z
+                except GeneratorExit:
+                    self._finalize()
+                    raise
 
 
-class Parmapper(Stream):
+class Parmapper(Iterable):
     def __init__(
         self,
-        instream: Stream,
+        instream: Iterable,
         func: Callable[[T], TT],
         *,
         executor: Literal["thread", "process"] = "process",
         num_workers: Optional[int] = None,
         return_x: bool = False,
         return_exceptions: bool = False,
+        executor_initializer=None,
+        executor_init_args=(),
         **kwargs,
     ):
-        super().__init__(instream)
-
-        if num_workers is None:
-            if executor == "thread":
-                num_workers = min(32, (os.cpu_count() or 1) + 4)
-            else:
-                num_workers = os.cpu_count() or 1
-        else:
-            assert num_workers > 0
+        assert executor in ("thread", "process")
+        if executor_initializer is None:
+            assert not executor_init_args
+        self._instream = instream
+        self._func = func
+        self._func_kwargs = kwargs
         self._return_x = return_x
         self._return_exceptions = return_exceptions
-        if executor == "thread":
-            self._executor = concurrent.futures.ThreadPoolExecutor(num_workers)
+        self._executor_ = executor
+        self._num_workers = num_workers
+        self._executor_initializer = executor_initializer
+        self._executor_init_args = executor_init_args
+        self._executor = None
+        self._finalize_func = None
+        self._loud_exception = False
+
+    def _start(self):
+        num_workers = self._num_workers
+        is_shared = False
+        if self._executor_ == "thread":
+            self._stopped = threading.Event()
+            if num_workers is None and self._executor_initializer is None:
+                self._executor = get_shared_thread_pool()
+                is_shared = True
+                num_workers = self._executor._max_workers
+            else:
+                if num_workers is None:
+                    num_workers = MAX_THREADS
+                else:
+                    assert 1 <= num_workers <= 100
+                self._executor = concurrent.futures.ThreadPoolExecutor(
+                    num_workers,
+                    initializer=self._executor_initializer,
+                    initargs=self._executor_init_args,
+                )
         else:
-            assert executor == "process"
             self._stopped = MP_SPAWN_CTX.Event()
-            self._executor = concurrent.futures.ProcessPoolExecutor(
-                num_workers, mp_context=MP_SPAWN_CTX
-            )
+            if num_workers is None and self._executor_initializer is None:
+                self._executor = get_shared_process_pool()
+                is_shared = True
+                num_workers = self._executor._max_workers
+            else:
+                if num_workers is None:
+                    num_workers = os.cpu_count() or 1
+                else:
+                    assert 1 <= num_workers <= (os.cpu_count() or 1) * 2
+                self._executor = concurrent.futures.ProcessPoolExecutor(
+                    num_workers,
+                    mp_context=MP_SPAWN_CTX,
+                    initializer=self._executor_initializer,
+                    initargs=self._executor_init_args,
+                )
         self._tasks = SingleLane(num_workers)
         self._worker = Thread(
-            target=self._start, args=(func,), kwargs=kwargs, loud_exception=False
+            target=self._run_worker,
+            args=(self._func,),
+            kwargs=self._func_kwargs,
+            loud_exception=self._loud_exception,
         )
         self._worker.start()
         self._finalize_func = Finalize(
             self,
-            type(self)._finalize,
-            (self._stopped, self._tasks, self._worker, self._executor),
+            type(self)._finalizer,
+            (
+                self._stopped,
+                self._tasks,
+                self._worker,
+                self._executor if is_shared else None,
+            ),
             exitpriority=10,
         )
 
-    def _start(self, func, **kwargs):
+    def _run_worker(self, func, **kwargs):
         tasks = self._tasks
-        while True:
-            try:
-                x = next(self._instream)
-            except StopIteration:
-                tasks.put(FINISHED)
-                break
-            except Exception:
-                tasks.put(STOPPED)
-                raise
-            if self._stopped.is_set():
-                break
-            t = self._executor.submit(func, x, **kwargs)
-            # The size of the queue `tasks` regulates how many
-            # concurrent calls to `func` there can be.
-            tasks.put((x, t))
-
-    @overrides
-    def _stop(self):
-        self._finalize_func()
+        executor = self._executor
+        try:
+            for x in self._instream:
+                if self._stopped.is_set():
+                    return
+                t = executor.submit(func, x, **kwargs)
+                tasks.put((x, t))
+                # The size of the queue `tasks` regulates how many
+                # concurrent calls to `func` there can be.
+        except Exception:
+            tasks.put(STOPPED)
+            raise
+        else:
+            tasks.put(FINISHED)
 
     @staticmethod
-    def _finalize(stopped, tasks, worker, executor):
+    def _finalizer(stopped, tasks, worker, executor):
         stopped.set()
         while not tasks.empty():
             _ = tasks.get()
         worker.join()
-        executor.shutdown()
+        if executor is not None:
+            executor.shutdown()
 
-    @overrides
-    def __next__(self):
-        if self._stopped.is_set():
-            raise StopIteration
-        z = self._tasks.get()
-        if z == FINISHED:
-            self._stop()
-            raise StopIteration
-        if z == STOPPED:
-            self._stop()
-            raise self._worker.exception()
+    def _finalize(self):
+        fin = self._finalize_func
+        if fin:
+            self._finalize_func = None
+            fin()
 
-        x, fut = z
-        try:
-            y = fut.result()
-            if self._return_x:
-                return x, y
-            return y
-        except Exception as e:
-            if self._return_exceptions:
-                # TODO: think about when `e` is a "remote exception".
-                if self._return_x:
-                    return x, e
-                return e
+    def __iter__(self):
+        self._start()
+
+        while True:
+            z = self._tasks.get()
+            if z == FINISHED:
+                break
+            if z == STOPPED:
+                raise self._worker.exception()
+
+            x, fut = z
+            try:
+                y = fut.result()
+            except Exception as e:
+                if self._return_exceptions:
+                    # TODO: think about when `e` is a "remote exception".
+                    try:
+                        if self._return_x:
+                            yield x, e
+                        else:
+                            yield e
+                    except GeneratorExit:
+                        self._finalize()
+                        raise
+                else:
+                    self._finalize()
+                    raise
             else:
-                self._stop()
-                raise
+                try:
+                    if self._return_x:
+                        yield x, y
+                    else:
+                        yield y
+                except GeneratorExit:
+                    self._finalize()
+                    raise
+
+
+Streamer = Stream
+"""An alias to :class:`Stream` for backward compatibility.
+
+.. deprecated:: 0.11.9
+    Will be removed in 0.12.2.
+    Use ``Stream`` instead.
+"""
