@@ -1,6 +1,6 @@
 """
-`multiprocessing`_ code can be tricky.
-``mpservice`` provides help to address several common trikeries.
+`multiprocessing`_ can be tricky.
+``mpservice`` provides help to address several common difficulties.
 
 First, it is a good idea to always use the non-default (on Linux) "spawn" method to start a process.
 :data:`~mpservice.util.MP_SPAWN_CTX` is provided to make this easier.
@@ -24,6 +24,7 @@ the traceback info will be lost in pickling. :class:`~mpservice.util.RemoteExcep
 """
 
 from __future__ import annotations
+
 import errno
 import functools
 import inspect
@@ -31,16 +32,20 @@ import logging
 import logging.handlers
 import multiprocessing
 import multiprocessing.connection
-import multiprocessing.queues
 import multiprocessing.context
+import multiprocessing.queues
 import os
 import subprocess
 import threading
 import traceback
+import warnings
+import weakref
+from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor
 from multiprocessing.util import Finalize
 from types import TracebackType
 from typing import Optional
 
+from deprecation import deprecated
 
 MAX_THREADS = min(32, (os.cpu_count() or 1) + 4)
 """
@@ -157,8 +162,9 @@ def get_docker_host_ip():
     return z[: z.find(" ")]
 
 
+@deprecated(deprecated_in="0.11.9", removed_in="0.12.2")
 def is_exception(e) -> bool:
-    # TODO: test showed the raised objects are always instances, not classes, even
+    # Test showed the raised objects are always instances, not classes, even
     # if we do
     #   raise ValueError
     # the captured object is a ValueError instance, not the class.
@@ -285,14 +291,14 @@ ValueError: 38
 >>>
 >>> err
 ValueError(38)
->>> err.__traceback__
-<traceback object at 0x7fad69dff8c0>
+>>> err.__traceback__  # doctest: +ELLIPSIS
+<traceback object at 0x7...>
 >>> err.__cause__ is None
 True
 >>>
 >>> e_remote = RemoteException(err)
->>> e_remote
-<mpservice.util.RemoteException object at 0x7fad702cd280>
+>>> e_remote  # doctest: +ELLIPSIS
+<mpservice.util.RemoteException object at 0x7...>
 >>> e_pickled = pickle.dumps(e_remote)
 >>> e_unpickled = pickle.loads(e_pickled)
 >>>
@@ -302,14 +308,24 @@ ValueError(38)
 <class 'ValueError'>
 >>> e_unpickled.__traceback__ is None
 True
->>> e_unpickled.__cause__
-RemoteTraceback('Traceback (most recent call last):\n  File "<stdin>", line 2, in <module>\n  File "<stdin>", line 2, in gee\n  File "<stdin>", line 2, in foo\nValueError: 38\n')
+>>> e_unpickled.__cause__  # doctest: +SKIP
+RemoteTraceback('Traceback (most recent call last):
+  File "<stdin>", line 2, in <module>
+  File "<stdin>", line 2, in gee
+  File "<stdin>", line 2, in foo
+ValueError: 38
+')
 >>>
 >>> is_remote_exception(e_unpickled)
 True
->>> get_remote_traceback(e_unpickled)
-'Traceback (most recent call last):\n  File "<stdin>", line 2, in <module>\n  File "<stdin>", line 2, in gee\n  File "<stdin>", line 2, in foo\nValueError: 38\n'
->>> print(get_remote_traceback(e_unpickled))
+>>> get_remote_traceback(e_unpickled)  # doctest: +SKIP
+'Traceback (most recent call last):
+  File "<stdin>", line 2, in <module>
+  File "<stdin>", line 2, in gee
+  File "<stdin>", line 2, in foo
+  ValueError: 38
+'
+>>> print(get_remote_traceback(e_unpickled))  # doctest: +SKIP
 Traceback (most recent call last):
     File "<stdin>", line 2, in <module>
     File "<stdin>", line 2, in gee
@@ -767,7 +783,7 @@ class SpawnProcess(multiprocessing.context.SpawnProcess):
 
     def run(self):
         """
-        Overrides the standard ``Process.run`.
+        Overrides the standard ``Process.run``.
 
         ``start`` arranges for this to be run in a child process.
         """
@@ -985,3 +1001,120 @@ class ProcessLogger:
             logging.getLogger().removeHandler(self._qh)
             self._q.close()
             self._q = None
+
+
+# References:
+#  https://thorstenball.com/blog/2014/10/13/why-threads-cant-fork/
+_global_thread_pools_: dict[str, ThreadPoolExecutor] = weakref.WeakValueDictionary()
+_global_process_pools_: dict[str, ProcessPoolExecutor] = weakref.WeakValueDictionary()
+_global_thread_pools_lock: threading.Lock = threading.Lock()
+_global_process_pools_lock: threading.Lock = threading.Lock()
+
+
+def get_shared_thread_pool(
+    name: str = "default", max_workers: int = None
+) -> ThreadPoolExecutor:
+    """
+    Get a globally shared "thread pool", that is,
+    `concurrent.futures.ThreadPoolExecutor <https://docs.python.org/3/library/concurrent.futures.html#concurrent.futures.ThreadPoolExecutor>`_.
+
+    This is often called with no argument, leaving both ``name`` and ``max_workers`` at their default.
+
+    The default value of ``name`` is "default".
+    Different values of ``name`` refer to independent executors.
+
+    The default value of ``max_workers`` is the default value of ThreadPoolExecutor.
+    (Since Python 3.8, it is ``min(32, (os.cpu_count() or 1) + 4)``.)
+    If an executor with the requested ``name`` does not exist, it will be created
+    with the specified ``max_workers`` argument (or using default if not specified).
+    However, if the name is "default", the internal default size is always used; user specified ``max_workers``,
+    if any, will be ignored.
+
+    If the named executor exists, it will be returned, and ``max_workers`` will be ignored.
+
+    User should assign the returned executor to a variable and keep the variable in scope
+    as long as the executor is needed.
+    Once all user references to a named executor (including the default one named "default")
+    have been garbage collected, the executor is gone. When it is requested again,
+    it will be created again.
+
+    User should not call ``shutdown`` on the returned executor.
+
+    This function is thread-safe, meaning it can be called safely in multiple threads with different
+    or the same ``name``.
+    """
+    with _global_thread_pools_lock:
+        executor = _global_thread_pools_.get(name)
+        # If the named pool exists, it is returned; the input `max_workers` is ignored.
+        if executor is None or executor._shutdown:
+            # `executor._shutdown` is True if user inadvertently called `shutdown` on the executor.
+            if name == "default":
+                if max_workers is not None:
+                    warnings.warn(
+                        f"size of the 'default' thread pool is determined internally; the input {max_workers} is ignored"
+                    )
+                    max_workers = None
+            else:
+                if max_workers is not None:
+                    assert 1 <= max_workers <= 64
+            executor = ThreadPoolExecutor(max_workers)
+            _global_thread_pools_[name] = executor
+    return executor
+
+
+def get_shared_process_pool(
+    name: str = "default", max_workers: int = None
+) -> ProcessPoolExecutor:
+    """
+    Get a globally shared "process pool", that is,
+    `concurrent.futures.ProcessPoolExecutor <https://docs.python.org/3/library/concurrent.futures.html#concurrent.futures.ProcessPoolExecutor>`_.
+
+    Analogous to :func:`get_shared_thread_pool`.
+    """
+    with _global_process_pools_lock:
+        executor = _global_process_pools_.get(name)
+        # If the named pool exists, it is returned; the input `max_workers` is ignored.
+        if executor is None or executor._processes is None:
+            # `executor._processes` is None if user inadvertently called `shutdown` on the executor.
+            if name == "default":
+                if max_workers is not None:
+                    warnings.warn(
+                        f"size of the 'default' process pool is determined internally; the input {max_workers} is ignored"
+                    )
+                    max_workers = None
+            else:
+                if max_workers is not None:
+                    assert 1 <= (os.cpu_count() or 1) * 2
+            executor = ProcessPoolExecutor(max_workers, mp_context=MP_SPAWN_CTX)
+            _global_process_pools_[name] = executor
+    return executor
+
+
+try:
+    register_at_fork = os.register_at_fork  # not available on Windows
+except AttributeError:  # on Windows
+    pass
+else:
+
+    def _clear_global_thread_process_pools():
+        for box in (_global_thread_pools_, _global_process_pools_):
+            for name in list(box.keys()):
+                pool = box.get(name)
+                if pool is not None:
+                    # TODO: if `pool` has locks, are there problems?
+                    pool.shutdown(wait=False)
+                box.pop(name, None)
+        global _global_thread_pools_lock
+        try:
+            _global_thread_pools_lock.release()
+        except RuntimeError:  # 'release unlocked lock'
+            pass
+        _global_thread_pools_lock = threading.Lock()
+        global _global_process_pools_lock
+        try:
+            _global_process_pools_lock.release()
+        except RuntimeError:
+            pass
+        _global_process_pools_lock = threading.Lock()
+
+    register_at_fork(after_in_child=_clear_global_thread_process_pools)
